import * as i0 from '@angular/core';
import { Pipe, NgModule } from '@angular/core';
import { pinyin } from 'pinyin-pro';

/**
 * @description: 检测汉语字符串和拼音是否匹配
 * @param {string} words 汉语字符串
 * @param {string} pinyin 拼音，支持各种缩写形式
 * @return {Array | null} 若匹配成功，返回拼音在汉字中的下标数组；若匹配失败，返回 null
 */
const matchOverride = (words, pinyin$1) => {
    const result = [];
    let currentPinyin = pinyin$1;
    let i = 0;
    // 是否为部分整体匹配
    let lowerWords = words.toLowerCase();
    let lowerPinyin = pinyin$1.toLowerCase();
    if (lowerWords.includes(lowerPinyin)) {
        i = lowerWords.indexOf(lowerPinyin);
    }
    for (; i < words.length; i++) {
        // 是否为一致匹配
        let x = lowerWords[i];
        let y = currentPinyin[0]?.toLowerCase();
        if (x === y) {
            currentPinyin = currentPinyin.slice(1);
            result.push(i);
            continue;
        }
        // 当前字的多音字拼音
        const ps = pinyin(words[i], {
            toneType: 'none',
            multiple: true,
            type: 'array',
        });
        let currentLength = 0;
        ps.forEach((p) => {
            const length = getMatchLength(p, currentPinyin);
            if (length > currentLength) {
                currentLength = length;
            }
        });
        if (currentLength) {
            currentPinyin = currentPinyin.slice(currentLength);
            result.push(i);
        }
        if (!currentPinyin) {
            break;
        }
    }
    return result.length && !currentPinyin ? result : null;
};
// 检测两个拼音最大的匹配长度
const getMatchLength = (pinyin1, pinyin2) => {
    let length = 0;
    for (let i = 0; i < pinyin1.length; i++) {
        if (pinyin1[i] === pinyin2[length]) {
            length++;
        }
    }
    return length;
};

const escape = (s) => s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
const replaceStartPattern = (s) => new RegExp(`^${s}`);
class HighlightPipe {
    /**
     * @name transform
     * @param value {string}
     * @param input {string}
     * @param pinyinMode {boolean} if enable pinyin parsing mode
     */
    transform(value, input, pinyinMode = false) {
        const trimmedInput = input?.trim();
        if (trimmedInput === null ||
            trimmedInput === undefined ||
            trimmedInput === '') {
            return value;
        }
        try {
            if (pinyinMode) {
                const m = matchOverride(value, input);
                if (m !== null) {
                    const pinyinArr = pinyin(value, {
                        nonZh: 'consecutive',
                        toneType: 'none',
                        v: true,
                        type: 'array',
                    });
                    if (pinyinArr.length) {
                        const firstCharArr = pinyin(value, {
                            pattern: 'first',
                            nonZh: 'consecutive',
                            toneType: 'none',
                            v: true,
                            type: 'array',
                        });
                        let newInputArr = [];
                        m.forEach((mi, i) => {
                            let regex = new RegExp(replaceStartPattern(pinyinArr[i]));
                            let newInput = input.replace(regex, '');
                            if (newInput === input) {
                                regex = new RegExp(replaceStartPattern(firstCharArr[i]));
                                newInput = input.replace(regex, '');
                            }
                            newInputArr.push(value[mi]);
                            input = newInput;
                        });
                        input = newInputArr.join('');
                    }
                }
            }
            const regex = new RegExp(`(${escape(input)})`, 'i');
            return value.replace(regex, '<b>$1</b>');
        }
        catch (e) {
            return value;
        }
    }
}
HighlightPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.9", ngImport: i0, type: HighlightPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
HighlightPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "14.2.9", ngImport: i0, type: HighlightPipe, name: "highlight" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.9", ngImport: i0, type: HighlightPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'highlight',
                }]
        }] });

class OdHighlightPipeModule {
}
OdHighlightPipeModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.9", ngImport: i0, type: OdHighlightPipeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
OdHighlightPipeModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.9", ngImport: i0, type: OdHighlightPipeModule, declarations: [HighlightPipe], exports: [HighlightPipe] });
OdHighlightPipeModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.9", ngImport: i0, type: OdHighlightPipeModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.9", ngImport: i0, type: OdHighlightPipeModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [HighlightPipe],
                    imports: [],
                    exports: [HighlightPipe],
                }]
        }] });

/*
 * Public API Surface of @linch90/od-highlight-pipe
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HighlightPipe, OdHighlightPipeModule };
//# sourceMappingURL=linch90-od-highlight-pipe.mjs.map
